require "benchmark"

module TweekMetrics
  class Handler
    attr_reader :instruments, :key

    def initialize(instruments, key, &block)
      @instruments = instruments
      @key = key
      @block = block

      handle!
    end

    def handle!
      if @block
        begin
          @real_time = Benchmark.measure { @block.call }.real
        rescue => e
          @exception = e
        end
      end

      if @exception
        TweekMetrics.statsd.increment(create_key(TweekMetrics.namespace, :global, :exception, @exception.class.name))
        TweekMetrics.statsd.increment(create_key(TweekMetrics.namespace, TweekMetrics.client, :exception, @exception.class.name))
      else
        if instruments.include?(:counter)
          TweekMetrics.statsd.increment(create_key(TweekMetrics.namespace, :global, key))
          TweekMetrics.statsd.increment(create_key(TweekMetrics.namespace, TweekMetrics.client, key))
        end

        if instruments.include?(:time)
          TweekMetrics.statsd.timing(create_key(TweekMetrics.namespace, :global, key), @real_time)
          TweekMetrics.statsd.timing(create_key(TweekMetrics.namespace, TweekMetrics.client, key), @real_time)
        end
      end
    end

    private

    def create_key(*items)
      items.flatten.compact.join(".")
    end
  end
end